/*
 *
 *  Linux Basic Security Modules
 *  LBSM project team. <lbsm@sec.jri.co.jp>
 *
 *  Copyright (c) 2002-2004 The Japan Research Institute, Limited
 *
 *  This is a part of Linux Basic Security Modules.
 *
 *  This programme is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This programme is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free software Foundation, Inc.,
 *  59 Temple Place, suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _LBSM_AUDIT_H
#define _LBSM_AUDIT_H

#include <linux/param.h>
#include <audit_kevent.h>
#include <audit_uevent.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 *  Memory utility function
 */
extern int memcpy_fromfs(char *, const char *, int);

/**
 *  Some typedefs for the fundamentals
 */
typedef int			aud_id_t;

/**
 *  Default data size
 */
#define AUDIT_BUF_LEN		10240
#define MAX_ABS_FILE_PATH	1024
#define MAX_HOSTNAME		1024
#define TIMEOUT			600

/**
 *  Auditd spec file
 */
#define AUDIT_SRC_PATH		"/dev/audit"
#define AUDIT_WRITE_SRC_PATH	"/dev/auditw"

/**
 *  CAP audit
 */
#define CAP_AUDIT_CONTROL       30

/**
 *  These are difinition for audit device driver's major version.
 */
#define LBSM_AUDIT_READ_DEV     63
#define LBSM_AUDIT_WRITE_DEV    64

/**
 *  auditon(2) commands
 */
#define A_SETKMASK		1    /* set kernel event preselection mask          */
#define A_SETCLASS		2    /* set audit event to class mapping            */ 
#define A_GETKMASK		3    /* get kernel event preselection mask          */
#define A_GETCLASS		4    /* get audit event to class mapping            */ 
#define A_SETKNMASK		5    /* set kernel event unauthed preselection mask */
#define A_GETKNMASK		6    /* set kernel event unauthed preselection mask */
#define A_SETUMASK		7    /* set preselection mask for procs with auid   */
#define A_GETUMASK		8    /* get preselection mask for procs with auid   */
#define A_SETPOLICY		9    /* set audit policy                            */
#define A_GETPOLICY		10   /* get audit policy                            */
#define A_INITIALIZE		11   /* initializing all mask                       */
#define A_SETPMASK		12   /* set preselection mask for an given pid      */
#define A_GETPMASK		13   /* get preselection mask for an given pid      */
#define A_GETCOND		14   /* get audit system on/off condition           */
#define A_SETCOND		15   /* set audit system on/off condition           */
#define A_GETQCTL		16   /* get kernel audit queue cntrol parameters    */
#define A_SETQCTL		17   /* set kernel audit queue cntrol parameters    */
#define A_GETCWD		18   /* get process current working directory       */
#define A_GETCAR		19   /* get process current active root             */
#define A_GETSTAT		20   /* get audit statistics                        */
#define A_SETSTAT		21   /* (re)set audit statistics                    */
#define A_SETSMASK		22   /* get preselection mask for procs with asid   */
#define A_GETPINFO		23   /* get audit info for an arbitrary pid         */
#define A_SETFSEZE		24   /* set audit file size                         */
#define A_GETFSEZE		25   /* get audit file size                         */

/**
 *   audit mask's
 */
extern unsigned long AUDIT_EVENT_MASK_SUCCESS;
extern unsigned long AUDIT_EVENT_MASK_FAILD;
extern unsigned long AUDIT_EVENT_NF_MASK_SUCCESS;
extern unsigned long AUDIT_EVENT_NF_MASK_FAILD;
extern unsigned long AUDIT_CLASS_MASK[MAX_UEVENTS];
extern unsigned long AUDIT_POLICY;
extern unsigned int  AUDIT_CONDITION;

/**
 *  Policy's mask
 */
#ifdef LBSM_SUPPORT_SELINUX
#define MAX_POLICY		12
#else
#define MAX_POLICY		13
#endif /* LBSM_SUPPORT_SELINUX */
extern unsigned long policy_mask[MAX_POLICY+1];

/**
 *  Audit policy
 */
#define AUDIT_ARGE		1    /* include arge with execv system call events  */
#define AUDIT_ARGV		2    /* include argv with execv system call events  */
#define AUDIT_CNT		3    /* do NOT sleep undelivered synchronous events */
#define AUDIT_GROUP		4    /* include group attribute with each record    */
#define AUDIT_PATH		5    /* allow multiple paths per event              */
#define AUDIT_TRAIL		6    /* include trailer token                       */
#define AUDIT_SEQ		7    /* DEBUG option: include sequence attribute    */
#define AUDIT_COMM		8    /* include command name with with each record  */
#define AUDIT_PARENT		9    /* include parent process information          */
#define AUDIT_AHLT		10   /* HALF machine on undelivered async event     */
#define AUDIT_PASSWD		11   /* include bad pasword with "login" event      */
#ifdef LBSM_SUPPORT_SELINUX
#define AUDIT_LABEL		12   /* include security label for SELinux          */
#endif /* LBSM_SUPPORT_SELINUX */

/**
 *  SELINUX sid type
 */
#ifdef LBSM_SUPPORT_SELINUX
#define	SELINUX_TASK		((short)0x01)
#define	SELINUX_INODE		((short)0x02)
#define	SELINUX_FILE		((short)0x03)
#define	SELINUX_SOCK		((short)0x04)
#define	SELINUX_IPC		((short)0x05)
#endif /* LBSM_SUPPORT_SELINUX */

/**
 * Audit conditions, statements reguarding what's to be done with
 * audit records.
 */
enum AUDIT_COND {
	/*disable,                      auditing is being done                      */
	noaudit,                     /* auditing is not being done                  */
	auditing                     /* audit module loaded but not enabled         */
};

/**
 *   A number of log record
 */
extern unsigned long   total_auth_log_count;
extern unsigned long   total_unauth_log_count;
extern unsigned long   total_audit_log_count;
extern unsigned long   total_log_size;
extern unsigned long   total_syscall_count;

/**
 *  An audit event mask.
 */
struct _aud_mask_n {
	unsigned int am_success;	/* success bits                            */
	unsigned int am_failure;	/* failure bits                            */
};
typedef struct _aud_mask_n aud_mask_t;

/**
 *   The structure of the terminal ID
 */
struct _aud_tid_n {
	short	driver;			/* driver ID                               */
	short	minor;			/* driver miner ID                         */
	char	machine[MAX_HOSTNAME];	/* machine name(it is not ip address)      */
};
typedef struct _aud_tid_n aud_tid_t;

/**
 *  Audit Information structure 
 */
struct _auditinfo_n {
	int		ai_auid;	/* audit ID                                */
	aud_mask_t	ai_mask;	/* user mask                               */
	int		ai_sid;		/* session ID                              */
	aud_tid_t	ai_tid;		/* terminal ID                             */
};
typedef struct _auditinfo_n auditinfo_t;

/**
 *  Audit stat structures
 */
struct _aud_stat_n {
        unsigned long	as_version;	/* version of kernel audit code            */
        unsigned long	as_numevent;	/* number of kernel audit events           */
        long		as_generated;	/* # records processed                     */
        long		as_nonattrib;	/* # non-attributed records produced       */
        long		as_kernel;	/* # records produced by kernel            */
        long		as_audit;	/* # records processed by audit(2)         */
        long		as_auditctl;	/* # records processed by auditctl(2)      */
        long		as_written;	/* # records written to audit trail        */
        long		as_totalsize;	/* total number bytes of audit data        */
        unsigned long	as_memused;     /* amount of memory used by auditing       */
};
typedef struct _aud_stat_n aud_stat_t;

/**
 *  Functions
 */
#ifdef __STDC__

extern int sys_auditon(unsigned int, aud_mask_t *, int);
extern int sys_auditstat(aud_stat_t *);
extern int sys_auditsvc(int, int);
extern int sys_audituser(char *);
extern int sys_getaudit(auditinfo_t *);
extern int sys_getaudit_t(auditinfo_t *, int);
extern int sys_getauid(aud_id_t);
extern int sys_kernstate(aud_mask_t *);
extern int sys_getuseraudit(aud_id_t, aud_mask_t *);
extern int sys_setaudit(auditinfo_t *);
extern int sys_setauid(aud_id_t);
extern int sys_setkernstate(aud_mask_t *);
extern int sys_setuseraudit(aud_id_t, aud_mask_t *);

#else

extern int sys_auditon();
extern int sys_auditstat();
extern int sys_auditsvc();
extern int sys_audituser();
extern int sys_getaudit();
extern int sys_getaudit_t();
extern int sys_getauid();
extern int sys_kernstate();
extern int sys_getuseraudit();
extern int sys_setaudit();
extern int sys_setauid();
extern int sys_setkernstate();
extern int sys_setuseraudit();

#endif /* __STDC__ */

#ifdef __cplusplus
}
#endif

#endif /* !_LBSM_AUDIT_H */
