/*
 *
 *  Linux Basic Security Modules
 *  LBSM project team. <lbsm@sec.jri.co.jp>
 *
 *  Copyright (c) 2002-2004 The Japan Research Institute, Limited
 *
 *  This is a part of Linux Basic Security Modules.
 *
 *  This programme is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This programme is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free software Foundation, Inc.,
 *  59 Temple Place, suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _LBSMA_IOCTL_H
#define _LBSMA_IOCTL_H

#include <linux/errno.h>
#include <linux/fs.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 *  This is definition for audit commands. In the LBSM, there are 14
 *  command for operating audit module.
 */

#define LBSMA_IOC_MAGIC	'l'

#define LBSMA_IOC_AUDITON	_IOWR(LBSMA_IOC_MAGIC,  1, unsigned long)
#define LBSMA_IOC_GET_AUDITSTAT	_IOWR(LBSMA_IOC_MAGIC,  2, unsigned long)
#define LBSMA_IOC_AUDITSVC	_IOWR(LBSMA_IOC_MAGIC,  3, unsigned long)
#define LBSMA_IOC_GET_AUDITUSER	_IOWR(LBSMA_IOC_MAGIC,  4, unsigned long)
#define LBSMA_IOC_GET_AUDIT	_IOWR(LBSMA_IOC_MAGIC,  5, unsigned long)
#define LBSMA_IOC_GET_AUID	_IOWR(LBSMA_IOC_MAGIC,  6, unsigned long)
#define LBSMA_IOC_GET_KERNSTATE	_IOWR(LBSMA_IOC_MAGIC,  7, unsigned long)
#define LBSMA_IOC_GET_USERAUDIT	_IOWR(LBSMA_IOC_MAGIC,  8, unsigned long)
#define LBSMA_IOC_SET_AUDIT	_IOWR(LBSMA_IOC_MAGIC,  9, unsigned long)
#define LBSMA_IOC_GET_AUID_T	_IOWR(LBSMA_IOC_MAGIC, 10, unsigned long)
#define LBSMA_IOC_SET_AUID	_IOWR(LBSMA_IOC_MAGIC, 11, unsigned long)
#define LBSMA_IOC_SET_USERAUDIT	_IOWR(LBSMA_IOC_MAGIC, 12, unsigned long)
#define LBSMA_IOC_SET_KERNSTATE	_IOWR(LBSMA_IOC_MAGIC, 13, unsigned long)
#define LBSMA_IOC_SET_MAC_LABEL	_IOWR(LBSMA_IOC_MAGIC, 14, unsigned long)
#define LBSMA_IOC_HARDRESET	_IO(LBSMA_IOC_MAGIC,   15)

#define LBSMA_IOC_MAXNR	15

/**
 *  This is a general structure for deriver some arguments to ioctl
 *  command. This structure has three arguments. The "cmd" is specified
 *  an audit command defined previously. The "arg1" includes five data
 *  structure. The "arg2" includes three data structure too. The "arg1"
 *  and "arg2"  present multiple data structure. 
 */
struct _audit_args_data_n {
	unsigned int cmd;

	union {
		auditinfo_t *ai;
		aud_mask_t  *mask;
		aud_stat_t  *st;
		char        *name;
		int         fd;
	} arg1;

	union {    
		int         pid;
		int         typ;
		aud_id_t    aid;
	} arg2;
};
typedef struct _audit_args_data_n audit_args_data_t;

/**
 *  Functions
 */
#ifdef __KERNEL__
#ifdef __STDC__

extern int audit_ioctl(struct inode *, struct file *, unsigned int, unsigned long);

#else

extern int audit_ioctl();

#endif /* __STDC__ */
#endif /* __KERNEL __ */

#ifdef __cplusplus
}
#endif

#endif /* !_LBSM_AUDIT_IOCTL_H */
