/*
 *
 *  Linux Basic Security Modules 
 *  LBSM project team. <lbsm@sec.jri.co.jp>
 *
 *  Copyright (c) 2002-2004 The Japan Research Institute, Limited
 *
 *  This is a part of Linux Basic Security Modules.
 *
 *  This programme is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This programme is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free software Foundation, Inc.,
 *  59 Temple Place, suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _LBSM_AUDIT_KERNEL_H
#define _LBSM_AUDIT_KERNEL_H

#include <linux/spinlock.h>
#include <asm/semaphore.h>
#include <audit.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __KERNEL__

/**
 *  This structure has some auditing informations. The elements of this structure
 *  is set by sys_fork or sys_execve or sys_clone. These informations is used
 *  to create audit tokens by kernel. 
 */
struct _p_audit_data_n {
	int              ai_auth;                     /* 1: authed 0: not authed  */
	int              ai_flg;                      /* if set mask or not       */
	int              ai_type;                     /* connection type          */
	uid_t            ai_auid;                     /* audit user ID            */
	int              ai_session;                  /* audit session ID         */
	aud_mask_t       ai_mask;                     /* user mask                */
	aud_mask_t       ai_smask;                    /* session mask             */
	aud_tid_t        ai_tid;                      /* terminal ID              */
	int		 ai_login_check_bit;	      /* login check bits         */
	struct semaphore sem;			      /* semaphore                */
};
typedef struct _p_audit_data_n p_audit_data_t;

/**
 *  This is max queue size. In LBSM, the kernel store audit record at AUD_MAXMEM
 *  bytes. You should not change this value. 
 */
#define AUD_MAXMEM       (1024 * 256)

/**
 *  This structure specifies audit queue. This queue store some audit record data
 *  and holds some queue status.
 */
struct _aud_queue_n {
	char              audit_buf[AUD_MAXMEM];      /* data                        */ 
	unsigned long     audit_start;                /* current position of buffer  */
	unsigned long     audit_chars;                /* number of written char      */
	unsigned long     audit_size;                 /* current size of buffer      */
	int               wt_block;                   /* writer is blocked           */
	int               rb_block;                   /* reader is blocked           */
	spinlock_t        lock;                       /* lock for queue modification */
};
typedef struct _aud_queue_n aud_queue_t;

extern aud_queue_t        log_queue;

/**
 *  Semaphore (this valiable is defined in audit.h)
 */
extern struct semaphore  audit_sem_level1;
extern wait_queue_head_t audit_wait;
extern wait_queue_head_t audit_wait2;

/**
 *  Memory cache statistics.
 */
struct _aud_list_stat_n {
	int              called;
	int              hit;
	int              miss;
	int              free;
	int              size;
};
typedef struct _aud_list_stat_n aud_list_stat_t;

#endif /* __KERNEL__ */

#ifdef __cplusplus
}
#endif

#endif  /* !_LBSM_AUDIT_KERNEL_H */
