/*
 *
 *  Linux Basic Security Modules 
 *  LBSM project team. <lbsm@sec.jri.co.jp>
 *
 *  Copyright (c) 2002-2004 The Japan Research Institute, Limited
 *
 *  This is a part of Linux Basic Security Modules.
 *
 *  This programme is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This programme is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free software Foundation, Inc.,
 *  59 Temple Place, suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _LBSM_AUDIT_RECORD_H
#define _LBSM_AUDIT_RECORD_H

#include <linux/fs.h>
#include <linux/in.h>
#include <audit.h>
#include <audit_info.h>
#include <audit_token.h>
#include <audit_kevent.h>
#include <audit_uevent.h>

#ifndef KERNEL_VERSION
#define KERNEL_VERSION(x,y,z) (((x)<<16)+((y)<<8)+(z))
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define  MAX_TOKEN		10
#define  MAX_BUF		1024
#define  LOCALHOST		"localhost"

/**
 *  MACROS
 */
#define CHECKPOLICY(type)                                                  \
	if ((AUDIT_POLICY & policy_mask[type]) == policy_mask[type])       \


#define ERR_STATE(type, err)                                               \
	if (err == NULL) {                                                 \
		freeToken(log_tt);                                         \
		printk("unable to get token %hd\n", type);                 \
		goto exit;                                                 \
	}                                                                  \

#define APPENDTOKEN(a, b, c)                                               \
	if ((appendToken(a, b, &c)) < 0) {                                 \
		freeToken(log_tt);                                         \
		printk("unable to get appendToken\n");                     \
		goto exit;                                                 \
	}                                                                  \

#define APPENDTOKEN_UP(a, b, c)                                            \
	if ((appendToken(a, b, &c)) < 0) {                                 \
		freeToken(log_tt);                                         \
		printk("unable to get appendToken\n");                     \
		up(&audit_sem_level1);                                     \
		goto exit;                                                 \
	}                                                                  \

/**
 *  Argument type
 */
#define AUD_ARGS_NONE		0x00000000      /* not specified                    */
#define AUD_ARG_PATH_VALUE	0x00000001	/* user space path name             */
#define AUD_ARG_PATH_VALUE_U	0x00000002      /* kernel space path name           */
#define AUD_ARG_ARGS_VALUE_C	0x00000004      /* character arguments              */
#define AUD_ARG_ARGS_VALUE_N	0x00000008      /* Number arguments                 */
#define AUD_ARG_MODE		0x00000010      /* mode arguments                   */
#define AUD_ARG_PATH_FROM_FD	0x00000100      /* create path name from fd         */
#define AUD_ARG_TEXT		0x00000200	/* create text token                */
#define AUD_ARG_INODE		0x10000000      /* create inode                     */
#define AUD_ARG_FD_PATH		0x20000000      /* create path name from fd         */

#define IS_ARG_INODE(m)         (((m) & AUD_ARG_INODE) == AUD_ARG_INODE)
#define IS_ARG_FD_PATH(m)       (((m) & AUD_ARG_FD_PATH) == AUD_ARG_FD_PATH)
#define AUD_GET_ORG_MODE(m)	((m) & ~(AUD_ARG_INODE|AUD_ARG_FD_PATH))

#ifdef __KERNEL__

/**
 *  Functions
 */
#ifdef __STDC__

extern int appendToken(token_t *,  token_t *, token_t **);
extern int checkMasks(int, int, int, int);
extern int checkOpenType(int);
extern int audit_record(int, struct inode *, struct task_struct *,
		           int, const char *, int, token_t *, 
		           int, int, struct ipc_perm *,
		           void *, struct in_addr *, 
		           u_short, int, token_t *, 
		           p_audit_data_t *, ...);
extern int setAuditFromLoginLog(struct task_struct *, token_t *);
extern int writeLoginLog(struct task_struct *, int, int);
extern int setCWD(struct task_struct *, const char *);
extern int setROOT(struct task_struct *, const char *);
extern void freeToken(token_t *);
extern int chkQueueSize(int);
extern int setUDPConnSession(struct task_struct *, struct socket *);
extern int setConnSession(struct task_struct *, struct socket *);
extern int unsetConnSession(struct task_struct *, struct inode *);
extern char *ndToFilename(struct nameidata *);
extern char *ndToFilenameFromFD(int);
extern struct inode *getInodeFromFD(int);
extern int getInodeFromFilename(const char *, struct inode *);
extern int org_user_path_walk(const char *, struct nameidata *, int);

#else

extern int appendToken();
extern int checkMasks();
extern int checkOpenType();
extern int audit_record();
extern int setAuditFromLoginLog();
extern int writeLoginLog();
extern int setCWD();
extern int setROOT();
extern void freeToken();
extern int chkQueueSize();
extern char *dentryToFilename();
extern int setUDPConnSession();
extern int setConnSession();
extern int unsetConnSession();
extern char *ndToFilename();
extern char *ndToFilenameFromFD();
extern struct inode *getInodeFromFD();
extern int getInodeFromFilename();
extern int org_user_path_walk();

#endif /* __STDC__  */

#endif

#ifdef __cplusplus
}
#endif

#endif  /* !_LBSM_AUDIT_RECORD_H */
