/*
 *
 *  Linux Basic Security Modules
 *  LBSM project team. <lbsm@sec.jri.co.jp>
 *
 *  Copyright (c) 2002-2004 The Japan Research Institute, Limited
 *
 *  This is a part of Linux Basic Security Modules.
 *
 *  This programme is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This programme is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free software Foundation, Inc.,
 *  59 Temple Place, suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _LBSM_AUDIT_RECORD_MAP_H
#define _LBSM_AUDIT_RECORD_MAP_H

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef __KERNEL__

#define MAX_TOKEN 10

/**
 *  Status of AUDIT MASK
 */
typedef enum _AUDIT_MASK {
	EVENT_MASK_SUCCESS,
	EVENT_MASK_FAILD
} AUDIT_MASK;

/**
 *  This structure defines event name and tokens type
 */
struct _audit_list_n {
	int    eventID;             /* audit event ID       */
	int    tokens[MAX_TOKEN];   /* mask of gettng token */
};

/**
 *  Definition of audit record
 */
static struct _audit_list_n audit_lists[]={
	{AUE_NULL,            {AUD_INVALID,}},
	{AUE_EXIT,            {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FORK,            {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN,            {AUD_PATH, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_CREAT,           {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_LINK,            {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_UNLINK,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_EXEC,            {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_CHDIR,           {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MKNOD,           {AUD_ARG, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_CHMOD,           {AUD_ARG, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_CHOWN,           {AUD_ARG, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_UMOUNT,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_JUNK,            {AUD_INVALID,}},
	{AUE_ACCESS,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_KILL,            {AUD_ARG, AUD_PROCESS, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_STAT,            {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_LSTAT,           {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_ACCT,            {AUD_ARG, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MCTL,            {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_REBOOT,          {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SYMLINK,         {AUD_TEXT, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_READLINK,        {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_EXECVE,          {AUD_PATH, AUD_ATTR, AUD_EXEC_ARGS, AUD_EXEC_ENV, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_CHROOT,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_VFORK,           {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETGROUPS,       {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETPGRP,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SWAPON,          {AUD_ARG, AUD_PATH, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETHOSTNAME,     {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FCNTL,           {AUD_ARG, AUD_SUBJECT, AUD_PATH, AUD_ATTR, AUD_INVALID,}},
	{AUE_SETPRIORITY,     {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_CONNECT,         {AUD_SOCKET, AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_ACCEPT,          {AUD_SOCKET, AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_BIND,            {AUD_SOCKET_INET, AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETSOCKOPT,      {AUD_SOCKET_INET, AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_VTRACE,          {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETTIMEOFDAY,    {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FCHOWN,          {AUD_ARG, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FCHMOD,          {AUD_ARG, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETREUID,        {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETREGID,        {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_RENAME,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_TRUNCATE,        {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FTRUNCATE,       {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FLOCK,           {AUD_PATH, AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SHUTDOWN,        {AUD_SOCKET_INET, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MKDIR,           {AUD_ARG, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_RMDIR,           {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_UTIMES,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_ADJTIME,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETRLIMIT,       {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_KILLPG,          {AUD_INVALID,}},
	{AUE_NFS_SVC,         {AUD_INVALID,}},
	{AUE_STATFS,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FSTATFS,         {AUD_ARG, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_UNMOUNT,         {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_ASYNC_DAEMON,    {AUD_INVALID,}},
	{AUE_NFS_GETFH,       {AUD_INVALID,}},
	{AUE_SETDOMAINNAME,   {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_QUOTACTL,        {AUD_INVALID,}},
	{AUE_EXPORTFS,        {AUD_INVALID,}},
	{AUE_MOUNT,           {AUD_ARG, AUD_TEXT, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMSYS,          {AUD_INVALID,}},
	{AUE_MSGSYS,          {AUD_INVALID,}},
	{AUE_SHMSYS,          {AUD_INVALID,}},
	{AUE_BSMSYS,          {AUD_INVALID,}},
	{AUE_RFSSYS,          {AUD_INVALID,}},
	{AUE_FCHDIR,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FCHROOT,         {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_VPIXSYS,         {AUD_INVALID,}},
	{AUE_PATHCONF,        {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN_R,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN_RC,         {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN_RT,         {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN_RTC,        {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN_W,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN_WC,	      {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN_WT, 	      {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN_WTC,        {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN_RW,         {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN_RWC,        {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN_RWT,	      {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OPEN_RWTC,       {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}}, 
	{AUE_MSGCTL,	      {AUD_INVALID,}},
	{AUE_MSGCTL_RMID,     {AUD_ARG, AUD_IPC, AUD_SUBJECT, AUD_INVALID,}}, 
	{AUE_MSGCTL_SET,      {AUD_ARG, AUD_IPC, AUD_SUBJECT, AUD_INVALID,}}, 
	{AUE_MSGCTL_STAT,     {AUD_ARG, AUD_IPC, AUD_SUBJECT, AUD_INVALID,}}, 
	{AUE_MSGGET,          {AUD_IPC, AUD_SUBJECT, AUD_INVALID,}}, 
	{AUE_MSGRCV,          {AUD_ARG, AUD_IPC, AUD_SUBJECT, AUD_INVALID,}}, 
	{AUE_MSGSND,	      {AUD_ARG, AUD_IPC, AUD_SUBJECT, AUD_INVALID,}}, 
	{AUE_SHMCTL,	      {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_INVALID,}},
	{AUE_SHMCTL_RMID,     {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}}, 
	{AUE_SHMCTL_SET,      {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}}, 
	{AUE_SHMCTL_STAT,     {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SHMGET,	      {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}}, 
	{AUE_SHMAT,  	      {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SHMDT, 	      {AUD_ARG, AUD_IPC, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMCTL,	      {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMCTL_RMID,     {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMCTL_SET,      {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMCTL_STAT,     {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMCTL_GETNCNT,  {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMCTL_GETPID,   {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMCTL_GETVAL,   {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMCTL_GETALL,   {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMCTL_GETZCNT,  {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMCTL_SETVAL,   {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMCTL_SETALL,   {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMGET,	      {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEMOP,           {AUD_ARG, AUD_IPC, AUD_IPC_PERM, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_CORE,            {AUD_INVALID,}},
	{AUE_CLOSE,           {AUD_ARG, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SYSTEMBOOT,      {AUD_TEXT, AUD_INVALID,}},
	{AUE_ASYNC_DAEMON_EXIT,{AUD_INVALID,}},
	{AUE_NFSSVC_EXIT,     {AUD_INVALID,}},
	{AUE_SETCMWPLABEL,    {AUD_INVALID,}},
	{AUE_SETCLEARANCE,    {AUD_INVALID,}},
	{AUE_FGETCMWLABEL,    {AUD_INVALID,}},
	{AUE_FSETCMWLABEL,    {AUD_INVALID,}},
	{AUE_GETCMWFSRANGE,   {AUD_INVALID,}},
	{AUE_GETCMWLABEL,     {AUD_INVALID,}},
	{AUE_GETFILEPRIV,     {AUD_INVALID,}},
	{AUE_LGETCMWLABEL,    {AUD_INVALID,}},
	{AUE_MKFSO,           {AUD_INVALID,}},
	{AUE_SETCMWLABEL,     {AUD_INVALID,}},
	{AUE_SETFILEPRIV,     {AUD_INVALID,}},
	{AUE_SETPROCPRIV,     {AUD_INVALID,}},
	{AUE_WRITEL,          {AUD_INVALID,}},
	{AUE_WRITEVL,         {AUD_INVALID,}},
	{AUE_GETAUID,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETAUID,         {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETAUDIT,        {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETAUDIT,        {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETUSERAUDIT,    {AUD_INVALID,}},
	{AUE_SETUSERAUDIT,    {AUD_INVALID,}},
	{AUE_AUDITSVC,        {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITUSER,       {AUD_INVALID,}},
	{AUE_AUDITON,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_GTERMID, {AUD_INVALID,}},
	{AUE_AUDITON_STERMID, {AUD_INVALID,}},
	{AUE_AUDITON_GPOLICY, {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_SPOLICY, {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_GESTATE, {AUD_INVALID,}},
	{AUE_AUDITON_SESTATE, {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_GQCTRL,  {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_SQCTRL,  {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETKERNSTATE,    {AUD_INVALID,}},
	{AUE_SETKERNSTATE,    {AUD_INVALID,}},
	{AUE_GETPORTAUDIT,    {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITSTAT,       {AUD_INVALID,}},
	{AUE_REVOKE,          {AUD_INVALID,}},
	{AUE_MAC,             {AUD_INVALID,}},
	{AUE_ENTERPROM,       {AUD_TEXT, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_EXITPROM,        {AUD_TEXT, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_IFLOAT,          {AUD_INVALID,}},
	{AUE_PFLOAT,          {AUD_INVALID,}},
	{AUE_UPRIV,           {AUD_INVALID,}},
	{AUE_IOCTL,           {AUD_PATH, AUD_SOCKET, AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FIND_RH,         {AUD_INVALID,}},
	{AUE_BADSATTR,        {AUD_INVALID,}},
	{AUE_TN_GEN,          {AUD_INVALID,}},
	{AUE_TFRWRD,          {AUD_INVALID,}},
	{AUE_TN_BYPASS,       {AUD_INVALID,}},
	{AUE_TN_ISPRIV,       {AUD_INVALID,}},
	{AUE_TN_CKRT,         {AUD_INVALID,}},
	{AUE_TN_CKIPOUT,      {AUD_INVALID,}},
	{AUE_KTNETD,          {AUD_INVALID,}},
	{AUE_STNETD,          {AUD_INVALID,}},
	{AUE_HLTSR,           {AUD_INVALID,}},
	{AUE_STRTSR,          {AUD_INVALID,}},
	{AUE_FREESR,          {AUD_INVALID,}},
	{AUE_TN_ACCRED,       {AUD_INVALID,}},
	{AUE_ONESIDE,         {AUD_INVALID,}},
	{AUE_MSGGETL,         {AUD_INVALID,}},
	{AUE_MSGRCVL,         {AUD_INVALID,}},
	{AUE_MSGSNDL,         {AUD_INVALID,}},
	{AUE_SEMGETL,         {AUD_INVALID,}},
	{AUE_SHMGETL,         {AUD_INVALID,}},
	{AUE_GETMLDADORN,     {AUD_INVALID,}},
	{AUE_GETSLDNAME,      {AUD_INVALID,}},
	{AUE_MLDLSTAT,        {AUD_INVALID,}},
	{AUE_MLDSTAT,         {AUD_INVALID,}},
	{AUE_SOCKET,          {AUD_INVALID,}},
	{AUE_SENDTO,          {AUD_INVALID,}},
	{AUE_PIPE,            {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SOCKETPAIR,      {AUD_INVALID,}},
	{AUE_SEND,            {AUD_INVALID,}},
	{AUE_SENDMSG,         {AUD_SOCKET, AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_RECV,            {AUD_INVALID,}},
	{AUE_RECVMSG,         {AUD_SOCKET, AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_RECVFROM,        {AUD_INVALID,}},
	{AUE_READ,            {AUD_PATH, AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETDENTS,        {AUD_ARG, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_LSEEK,           {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_WRITE,           {AUD_PATH, AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_WRITEV,          {AUD_PATH, AUD_ARG, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_NFS,             {AUD_INVALID,}},
	{AUE_READV,           {AUD_PATH, AUD_ARG, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OSTAT,           {AUD_INVALID,}},
	{AUE_SETUID,          {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_STIME,           {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_UTIME,           {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_NICE,            {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OSETPGRP,        {AUD_INVALID,}},
	{AUE_SETGID,          {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_READL,           {AUD_INVALID,}},
	{AUE_READVL,          {AUD_INVALID,}},
	{AUE_FSTAT,           {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_DUP2,            {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MMAP,            {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDIT,           {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_PRIOCNTLSYS,     {AUD_INVALID,}},
	{AUE_MUNMAP,          {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETEGID,         {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETEUID,         {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_PUTMSG,          {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETMSG,          {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_PUTPMSG,         {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETPMSG,         {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITSYS,        {AUD_INVALID,}},
	{AUE_AUDITON_GETKMASK,{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_SETKMASK,{AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_GETCWD,  {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_GETCAR,  {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_GETSTAT, {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_SETSTAT, {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_SETUMASK,{AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_SETSMASK,{AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_GETCOND, {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_SETCOND, {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_GETCLASS,{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_AUDITON_SETCLASS,{AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FUSERS,          {AUD_INVALID,}},
	{AUE_STATVFS,         {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_XSTAT,           {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_LXSTAT,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_LCHOWN,          {AUD_ARG, AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MEMCNTL,         {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SYSINFO,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_XMKNOD,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FORK1,           {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MODCTL,          {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MODLOAD,         {AUD_TEXT, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MODUNLOAD,       {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MODCONFIG,       {AUD_TEXT, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MODADDMAJ,       {AUD_TEXT, AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SOCKACCEPT,      {AUD_INVALID,}},
	{AUE_SOCKCONNECT,     {AUD_INVALID,}},
	{AUE_SOCKSEND,        {AUD_INVALID,}},
	{AUE_SOCKRECEIVE,     {AUD_INVALID,}},
	{AUE_REUGID,          {AUD_INVALID,}},
	{AUE_SETFSUID,        {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETFSGID,        {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETHOSTNAME,     {AUD_PATH, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETDOMAINNAME,   {AUD_INVALID,}},
	{AUE_UNAME,           {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETSOCKOPT,      {AUD_ARG, AUD_SOCKET_INET, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_LISTEN,          {AUD_ARG, AUD_SOCKET_INET, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SELECT,          {AUD_SOCKET_INET, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SWAPOFF,         {AUD_PATH, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SELECT2,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEC_ALERT,       {AUD_TEXT, AUD_INVALID,}},
	{AUE_ALARM,           {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETSID,          {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETSID,          {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETPRIORITY,     {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_IOPERM,          {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_IOPL,            {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MODIFY_LDT,      {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_CREATE_MODULE,   {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_INIT_MODULE,     {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_DELETE_MODULE,   {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GET_KERNEL_SYMS, {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_PAUSE,           {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_PERSONALITY,     {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETGROUPS,       {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETITIMER,       {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETRLIMIT,       {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETRUSAGE,       {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SYSCTL,          {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETTIMEOFDAY,    {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_TIMES,           {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_WAIT4,           {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_BDFLUSH,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_LLSEEK,          {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_READDIR,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SYNC,            {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FSYNC,           {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FDATASYNC,       {AUD_PATH, AUD_ATTR, AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SYSFS,           {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_USELIB,          {AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_UMASK,           {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_VHANGUP,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MPROTECT,        {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MREMAP,          {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MSYNC,           {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETPID,          {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETPPID,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETUID,          {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETGID,          {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETEGID,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETEUID,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETPGID,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETPGID,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETPGRP,         {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_DUP,             {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_PREAD,           {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_PWRITE,          {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEC_VIOLATE,     {AUD_TEXT, AUD_INVALID,}},
	{AUE_SIGNAL,          {AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_USTAT,           {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SCHED_SETPARAM,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SCHED_SETSCHEDULER,{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETRESUID,		{AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_PRCTL,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_CAPSET,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_TRUNCATE64,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FTRUNCATE64,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_STAT64,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_LSTAT64,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FSTAT64,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_LCHOWN32,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETREUID32,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETREGID32,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETGROUPS32,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FCHOWN32,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETRESUID32,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETRESGID32,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_CHOWN32,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETUID32,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETGID32,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETGID32,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETFSUID32,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETFSGID32,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_PIVOT_ROOT,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_FCNTL64,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETRESGID,		{AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_PTRACE,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_TIME,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_BRK,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_OLDUNAME,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SIGACTION,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SGETMASK,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SSETMASK,		{AUD_ARG, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SIGSUSPEND,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SIGPENDING,	{AUD_SUBJECT, AUD_INVALID,}},	
	{AUE_MINCORE,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MADVISE,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETRESUID,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETRESGID,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SYSLOG,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SETITIMER,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_VM86OLD,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_IPC,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SIGRETURN,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_NEWUNAME,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SIGPROCMASK,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MLOCK,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MUNLOCK,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MLOCKALL,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_MUNLOCKALL,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SCHED_GETPARAM,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SCHED_GETSCHEDULER,{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SCHED_YIELD,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SCHED_GET_PRIORITY_MAX,{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SCHED_GET_PRIORITY_MIN,{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SCHED_RR_GET_INTERVAL, {AUD_SUBJECT, AUD_INVALID,}},
	{AUE_NANOSLEEP,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_VM86,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_QUERY_MODULE,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_POLL,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_NFSSERVCTL,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_RT_SIGRETURN,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_RT_SIGACTION,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_RT_SIGPROCMASK,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_RT_SIGPENDING,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_RT_SIGTIMEDWAIT,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_RT_SIGQUEUEINFO,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_RT_SIGSUSPEND,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_GETCWD,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_CAPGET,		{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SENDFILE,		{AUD_SUBJECT, AUD_INVALID,}},
#ifdef LBSM_SUPPORT_SELINUX
	{AUE_SEL_TRANSITION_SID,{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEL_MEMBER_SID,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEL_SID_TO_CONTEXT,{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEL_CONTEXT_TO_SID,{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEL_LOAD_POLICY,	{AUD_PATH, AUD_ATTR, AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEL_CHANGE_SID,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEL_GET_SIDS,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEL_GET_USER_SIDS,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEL_AVC_TOGGLE,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEL_GETOSECSID,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEL_GETSECSID,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEL_AVC_ENFORCING,	{AUD_SUBJECT, AUD_INVALID,}},
	{AUE_SEL_MLS,		{AUD_SUBJECT, AUD_INVALID,}},
#endif /* LBSM_SUPPORT_SELINUX */
	{-1,			{AUD_INVALID,}}   
};

#define UEVENT_BASE         AUE_at_create

static struct _audit_list_n audit_user_elists[]={
	{AUE_at_create,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_GROUPS, AUD_INVALID,}},
	{AUE_at_delete,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_GROUPS, AUD_INVALID,}},
	{AUE_at_perm,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_GROUPS, AUD_INVALID,}},
	{AUE_cron_invoke,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_crontab_create,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_GROUPS, AUD_INVALID,}},
	{AUE_crontab_delete,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_GROUPS, AUD_INVALID,}},
	{AUE_crontab_perm,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_GROUPS, AUD_INVALID,}},
	{AUE_inetd_connect,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_login,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_logout,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_telnet,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_rlogin,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_mountd_mount,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_mountd_umount,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_rshd,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_su,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_halt_linux,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_reboot_linux,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_rexecd,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_passwd,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_rexd,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_ftpd,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_allocate_succ,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_allocate_fail,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_deallocate_succ,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_deallocate_fail,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_listdevice_succ,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_listdevice_fail,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_sshlogin,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_xlogin,		{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{AUE_security_alert,	{AUD_SUBJECT, AUD_TEXT, AUD_PATH, AUD_INVALID,}},
	{-1,			{AUD_INVALID,}}
};

#endif /* __KERNEL__ */

#ifdef __cplusplus
}
#endif

#endif /*  !_LBSM_AUDIT_RECORD_MAP_H  */
