/*
 *
 *  Linux Basic Security Modules
 *  LBSM project team. <lbsm@sec.jri.co.jp>
 *
 *  Copyright (c) 2002-2004 Japan Research Institute, Limited
 *
 *  This is a part of Linux Basic Security Modules.
 *
 *  This programme is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This programme is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free software Foundation, Inc.,
 *  59 Temple Place, suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _LBSM_AUDIT_SYSCALLS_H
#define _LBSM_AUDIT_SYSCALLS_H

#include <linux/sys.h>
#include <asm/semaphore.h>
#include <audit_record.h>

#ifdef  __cplusplus
extern "C" {
#endif

#ifdef __KERNEL__

/**
 *  This structure presents replaced system calls. The audit module initialize 
 *  this structure when installing to kernel area.
 */
struct _audit_intercept_n {
	int			is_replace;	/* flag for replace or noreplace          */
	void *			fn;		/* audit system call                      */
	struct semaphore	sem;		/* per-syscall semaphore   (not used)     */
};
typedef struct _audit_intercept_n audit_intercept_t;

/**
 * __aud_syscalln
 *	macro to generate an audit wrapper for a n argument syscall
 * 
 * returns
 *	whatever is indicated in typ
 * 
 * arguments
 *	syscall	- syscall number (NOT function)
 *	sysname - syscall name (e.g. _NR_setuid is really sys_setuid16)
 *	typ	- return type
 *	typ1	- first argument type of system call
 *	arg1	- first argument of system call
 *	tok1	- token id representing the first argument
 *      ...	- you get the point...
 */

#define __audit_syscall0(syscall,sysname,aname,typ) \
asmlinkage typ __audit_##sysname (void) \
{\
	long ret;\
	asmlinkage typ (*fn) (void);\
	fn = orig_sys_call_table[__NR_##syscall];\
	ret = fn ();\
	audit_record(aname,NULL,current,ret,NULL,0,NULL,0,0,NULL,NULL,NULL,0,0,NULL,NULL,AUE_NULL);\
	return(ret);\
}

#define __audit_syscall1(syscall,sysname,aname,typ,typ1,arg1,atyp1,kwd1) \
asmlinkage typ __audit_##sysname (typ1 arg1) \
{\
	long ret;\
	asmlinkage typ (*fn) (typ1);\
	fn = orig_sys_call_table[__NR_##syscall];\
	ret = fn (arg1);\
	audit_record(aname,NULL,current,ret,NULL,0,NULL,0,0,NULL,NULL,NULL,0,0,NULL,NULL,atyp1,(void *)&(arg1),(char *)kwd1, AUE_NULL);\
	return(ret);\
}

#define __audit_syscall2(syscall,sysname,aname,typ,typ1,arg1,typ2,arg2,atyp1,atyp2,kwd1,kwd2) \
asmlinkage typ __audit_##sysname (typ1 arg1, typ2 arg2) \
{\
	long ret;\
	asmlinkage typ (*fn) (typ1,typ2);\
	fn = orig_sys_call_table[__NR_##syscall];\
	ret = fn ((arg1),(arg2));\
	audit_record(aname,NULL,current,ret,NULL,0,NULL,0,0,NULL,NULL,NULL,0,0,NULL,NULL,atyp1,(void *)&(arg1),(char *)kwd1,atyp2,(void *)&(arg2),(char *)kwd2,AUE_NULL);\
	return(ret);\
}

#define __audit_syscall3(syscall,sysname,aname,typ,typ1,arg1,typ2,arg2,typ3,arg3,atyp1,atyp2,atyp3,kwd1,kwd2,kwd3) \
asmlinkage typ __audit_##sysname (typ1 arg1, typ2 arg2, typ3 arg3) \
{\
	long ret;\
	asmlinkage typ (*fn) (typ1,typ2,typ3);\
	fn = orig_sys_call_table[__NR_##syscall];\
	ret = fn ((arg1),(arg2),(arg3));\
	audit_record(aname,NULL,current,ret,NULL,0,NULL,0,0,NULL,NULL,NULL,0,0,NULL,NULL,atyp1,(void *)&(arg1),(char *)kwd1,atyp2,(void *)&(arg2),(char *)kwd2,atyp3,(void *)&(arg3),(char *)kwd3,AUE_NULL); \
	return(ret);\
}

#define __audit_syscall4(syscall,sysname,aname,typ,typ1,arg1,typ2,arg2,typ3,arg3,typ4,arg4,atyp1,atyp2,atyp3,atyp4,kwd1,kwd2,kwd3,kwd4) \
asmlinkage typ __audit_##sysname (typ1 arg1, typ2 arg2, typ3 arg3, typ4 arg4) \
{\
	long ret;\
	asmlinkage typ (*fn) (typ1,typ2,typ3,typ4);\
	fn = orig_sys_call_table[__NR_##syscall];\
	ret = fn ((arg1),(arg2),(arg3),(arg4));\
	audit_record(aname,NULL,current,ret,NULL,0,NULL,0,0,NULL,NULL,NULL,0,0,NULL,NULL,atyp1,(void *)&(arg1),(char *)kwd1,atyp2,(void *)&(arg2),(char *)kwd2,atyp3,(void *)&(arg3),(char *)kwd3,atyp4,(void *)&(arg4),(char *)kwd4,AUE_NULL); \
	return(ret);\
}

#define __audit_syscall5(syscall,sysname,aname,typ,typ1,arg1,typ2,arg2,typ3,arg3,typ4,arg4,typ5,arg5,atyp1,atyp2,atyp3,atyp4,atyp5,kwd1,kwd2,kwd3,kwd4,kwd5) \
asmlinkage typ __audit_##sysname (typ1 arg1, typ2 arg2, typ3 arg3, typ4 arg4, typ5 arg5) \
{\
	long ret;\
	asmlinkage typ (*fn) (typ1,typ2,typ3,typ4,typ5);\
	fn = orig_sys_call_table[__NR_##syscall];\
	ret = fn ((arg1),(arg2),(arg3),(arg4),(arg5));\
	audit_record(aname,NULL,current,ret,NULL,0,NULL,0,0,NULL,NULL,NULL,0,0,NULL,NULL,atyp1,(void *)&(arg1),(char *)kwd1,atyp2,(void *)&(arg2),(char *)kwd2,atyp3,(void *)&(arg3),(char *)kwd3,atyp4,(void *)&(arg4),(char *)kwd4,atyp5,(void *)&(arg5),(char *)kwd5,AUE_NULL); \
	return(ret);\
}

#define __audit_syscall6(syscall,sysname,aname,typ,typ1,arg1,typ2,arg2,typ3,arg3,typ4,arg4,typ5,arg5,typ6,arg6,atyp1,atyp2,atyp3,atyp4,atyp5,atyp6,kwd1,kwd2,kwd3,kwd4,kwd5,kwd6) \
asmlinkage typ __audit_##sysname (typ1 arg1, typ2 arg2, typ3 arg3, typ4 arg4, typ5 arg5, typ6 arg6) \
{\
	long ret;\
	asmlinkage typ (*fn) (typ1,typ2,typ3,typ4,typ5,typ6);\
	fn = orig_sys_call_table[__NR_##syscall];\
	ret = fn ((arg1),(arg2),(arg3),(arg4),(arg5),(arg6));\
	audit_record(aname,NULL,current,ret,NULL,0,NULL,0,0,NULL,NULL,NULL,0,0,NULL,NULL,atyp1,(void *)&(arg1),(char *)kwd1,atyp2,(void *)&(arg2),(char *)kwd2,atyp3,(void *)&(arg3),(char *)kwd3,atyp4,(void *)&(arg4),(char *)kwd4,atyp5,(void *)&(arg5),(char *)kwd5,atyp6,(void *)&(arg6),(char *)kwd6,AUE_NULL); \
	return(ret);\
}

/**
 * REPLACE_SYSCALL
 *      this macro inserts an audit wrapper into the audit syscall
 *      table for the given (you guessed it) syscall
 */
#define REPLACE_SYSCALL(syscall,sysname)\
	orig_sys_call_table[__NR_##syscall] = sys_call_table[__NR_##syscall];\
	audit_sys_call_table[__NR_##syscall].fn = __audit_##sysname;\
	audit_sys_call_table[__NR_##syscall].is_replace = 1;\
	sema_init(&(audit_sys_call_table[__NR_##syscall].sem), 1);

/**
 *  Global variable
 */
extern void *sys_call_table[];

/**
 *  Funcstions
 */
#ifdef __STDC__

extern void audit_intercept_syscalls(void);
extern void audit_restore_syscalls(void);

#else

extern void audit_intercept_syscalls();
extern void audit_restore_syscalls();

#endif /* __STDC__ */

#endif /* __KERNEL__ */

#ifdef  __cplusplus
}
#endif

#endif  /* !_LBSM_AUDIT_SYSCALLS_H */
