/*
 *
 *  Linux Basic Security Modules
 *  LBSM project team. <lbsm@sec.jri.co.jp>
 *
 *  Copyright (c) 2002-2004 The Japan Research Institute, Limited
 *
 *  This is a part of Linux Basic Security Modules.
 *
 *  This programme is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This programme is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free software Foundation, Inc.,
 *  59 Temple Place, suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _LBSM_AUDIT_TOKEN_H
#define _LBSM_AUDIT_TOKEN_H

#ifdef __KERNEL__
#include <linux/in.h>
#include <linux/types.h>
#include <asm/byteorder.h>
#ifdef LBSM_SUPPORT_SELINUX
#include <linux/flask/flask.h>
#endif /* LBSM_SUPPORT_SELINUX */
#else
#include <linux/limits.h>
#ifdef LBSM_SUPPORT_SELINUX
typedef char*	security_context_t;
#endif /* LBSM_SUPPORT_SELINUX */
#endif /* __KERNEL__ */

#include <audit_version.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 *  Event data type
 */
typedef short			aud_event_t;

/**
 *  Version of audit attribute
 */
#define MAX_ARGS_LEN		128

/**
 *  default data size
 */
#define INT_SIZE		32
#define MAX_SOCK_ADDR		128
#define BUF_SIZE		1000
#define TOKEN_BUF_SIZE		1024
#define MAX_TOKEN_SIZE		40

#define AUD_ARG_LEN		((AUR_BYTE*2)+AUR_SHORT+AUR_LONG)	
#define AUD_ATTR_LEN		(AUR_BYTE+(AUR_SHORT*5)+AUR_LONG)
#define AUD_EXEC_ARGS_LEN	(AUR_BYTE+AUR_LONG)	
#define AUD_EXEC_ENV_LEN	(AUR_BYTE+AUR_LONG)	
#define AUD_EXIT_LEN		(AUR_BYTE+(AUR_LONG*2))
#define AUD_HEADER_LEN		((AUR_BYTE*2)+(AUR_LONG*3)+(AUR_SHORT*2))
#define AUD_IN_ADDR_LEN		(AUR_BYTE+AUR_LONG)	
#define AUD_IP_LEN		(AUR_BYTE+AUR_IPHDR)
#define AUD_IPC_LEN		((AUR_BYTE*2)+AUR_LONG)
#define AUD_IPC_PERM_LEN	(AUR_BYTE+(AUR_SHORT*6)+AUR_LONG)
#define AUD_IPORT_LEN		(AUR_BYTE+AUR_SHORT) 
#define AUD_GROUPS_LEN		(AUR_BYTE+AUR_SHORT)
#define AUD_OPAQUE_LEN		(AUR_BYTE+AUR_SHORT)
#define AUD_PATH_LEN		(AUR_BYTE+AUR_SHORT)
#define AUD_PROCESS_LEN		((AUR_BYTE*2)+(AUR_LONG*3)+(AUR_SHORT*6))
#define AUD_RETURN_LEN		((AUR_BYTE*2)+AUR_LONG)
#define AUD_SEQ_LEN		(AUR_BYTE+AUR_LONG)
#define AUD_SUBJECT_LEN		(AUR_BYTE+(AUR_LONG*3)+(AUR_SHORT*7))
#define AUD_SOCKET_LEN		(AUR_BYTE+(AUR_LONG*2)+(AUR_SHORT*3))
#define AUD_SOCKET_INET_LEN	(AUR_BYTE+AUR_LONG+(AUR_SHORT*2))
#define AUD_TEXT_LEN		(AUR_BYTE+AUR_SHORT)
#define AUD_TRAILER_LEN		(AUR_BYTE+AUR_LONG+AUR_SHORT)
#define AUD_COMM_LEN		(AUR_BYTE+AUR_SHORT)
#define AUD_PARENT_LEN		(AUR_BYTE+(AUR_SHORT*2))
#define AUD_UNIX_DOMAIN_LEN	(AUR_BYTE+AUR_LONG+(AUR_SHORT*2))

/**
 *  Audit record token type codes
 */

/**
 * Control token type
 */
#define	AUD_INVALID		((char) 0x00)
#define	AUD_OTHER_FILE		((char) 0x11)
#define	AUD_OHEADER		((char) 0x12)
#define	AUD_TRAILER		((char) 0x13)
#define	AUD_HEADER		((char) 0x14)
#define	AUD_TRAILER_MAGIC	((short) 0xB105)

/**
 *  Data token types
 */
#define	AUD_DATA		((char) 0x21)
#define	AUD_IPC			((char) 0x22)
#define	AUD_PATH		((char) 0x23)
#define	AUD_SUBJECT		((char) 0x24)
#define	AUD_SERVER		((char) 0x25)
#define	AUD_PROCESS		((char) 0x26)
#define	AUD_RETURN		((char) 0x27)
#define	AUD_TEXT		((char) 0x28)
#define	AUD_OPAQUE		((char) 0x29)
#define	AUD_IN_ADDR		((char) 0x2A)
#define	AUD_IP			((char) 0x2B)
#define	AUD_IPORT		((char) 0x2C)
#define	AUD_ARG			((char) 0x2D)
#define	AUD_SOCKET		((char) 0x2E)
#define	AUD_SEQ			((char) 0x2F)

/**
 *  Modifier token types
 */
#define	AUD_ATTR		((char) 0x31)
#define	AUD_IPC_PERM		((char) 0x32)
#define	AUD_LABEL		((char) 0x33)
#define	AUD_GROUPS		((char) 0x34)
#define	AUD_ILABEL		((char) 0x35)
#define	AUD_SLABEL		((char) 0x36)
#define	AUD_CLEAR		((char) 0x37)
#define	AUD_PRIV		((char) 0x38)
#define	AUD_UPRIV		((char) 0x39)
#define	AUD_LIAISON		((char) 0x3A)
#define	AUD_NEWGROUPS		((char) 0x3B)
#define	AUD_EXEC_ARGS		((char) 0x3C)
#define	AUD_EXEC_ENV		((char) 0x3D)
#define AUD_SOCKET_INET		((char) 0x3E)

/**
 *  Command token types
 */
#define	AUD_CMD   		((char) 0x51)
#define	AUD_EXIT   		((char) 0x52)
#define	AUD_ACL			((char) 0x60)

/**
 *  LBSM Original token
 */
#define AUD_COMM		((char) 0x70)
#define AUD_PARENT		((char) 0x71)
#define AUD_UNIX_DOMAIN		((char) 0x72)

/**
 *  Audit print suggestion types.
 */
#define	AUP_BINARY	        ((char) 0)
#define	AUP_OCTAL	        ((char) 1)
#define	AUP_DECIMAL	        ((char) 2)
#define	AUP_HEX		        ((char) 3)
#define	AUP_STRING	        ((char) 4)

/**
 *  Audit data member types.
 */
#define	AUR_BYTE	        ((char) 1)
#define	AUR_CHAR	        ((char) 1)
#define	AUR_SHORT	        ((char) 2)
#define	AUR_INT		        ((char) 2)
#define	AUR_LONG	        ((char) 4)
#define AUR_IPHDR               ((char)21)

/**
 *  audit token IPC types (shm, sem, msg) [for ipc attribute]
 */
#define	AT_IPC_MSG	        ((char) 1)	/* message IPC id       */
#define	AT_IPC_SEM	        ((char) 2)	/* semaphore IPC id     */
#define	AT_IPC_SHM	        ((char) 3)	/* shared memory IPC id */

#ifndef __KERNEL__

/**
 *    token structure
 */
struct token_id_t {
	char  token_id;				/* token id   */
	char  token_name[TOKEN_BUF_SIZE];	/* token name */
};

/**
 *  Audit record token name
 */
#ifdef __TOKEN_ID_LIST
static struct token_id_t token_id_list [] = {
	{AUD_OTHER_FILE,	"file"},
	{AUD_OHEADER,		"oheader"},
	{AUD_TRAILER,		"trailer"},
	{AUD_HEADER,		"header"},
	{AUD_DATA,		"data"},
	{AUD_IPC,		"IPC"},
	{AUD_PATH,		"path"},
	{AUD_SUBJECT,		"subject"},
	{AUD_SERVER,		"server"},
	{AUD_PROCESS,		"process"},
	{AUD_RETURN,		"return"},
	{AUD_TEXT,		"text"},
	{AUD_OPAQUE,		"opaque"},
	{AUD_IN_ADDR,		"ip addr"},
	{AUD_IP,		"ip address"},
	{AUD_IPORT,		"ip port"},
	{AUD_ARG,		"argument"},
	{AUD_SOCKET,		"socket"},
	{AUD_SEQ,		"sequence"},
	{AUD_ATTR,		"attribute"},
	{AUD_IPC_PERM, 		"IPC perm"},
	{AUD_LABEL, 		"label"},
	{AUD_GROUPS, 		"group"},
	{AUD_ILABEL,		"ilabel"},
	{AUD_SLABEL,		"slabel"},
	{AUD_CLEAR,		"clear"},
	{AUD_PRIV,		"priv"},
	{AUD_UPRIV,		"upriv"},
	{AUD_LIAISON,		"liaison"},
	{AUD_NEWGROUPS,		"newgroups"},
	{AUD_EXEC_ARGS,		"exec_args"},
	{AUD_EXEC_ENV,		"exec_env"},
	{AUD_SOCKET_INET,	"socket-inet"},
	{AUD_CMD,		"cmd"},
	{AUD_EXIT,		"exit"},
	{AUD_ACL,		"acl"},
	{AUD_COMM,		"comm"},
	{AUD_PARENT,		"parent"},
	{AUD_UNIX_DOMAIN,	"unix domain"},
	{AUD_INVALID,		"invalid"}
};
#endif /* !__TOKEN_ID_LIST */

#endif /* __KERNEL__ */
 
#ifdef __KERNEL__

#define MAX_ARG_LEN             1024

/**
 *  token buffer
 */
struct _token_n {
	long		 tt_size;		/* Size of data */
	char *		 tt_data;		/* The data     */
	struct token_n * next;			/* Next token   */
};
typedef struct _token_n token_t;

/**
 *  Macros
 */
#define aud_buff_init(max_length, tt)                                    \
        tt = (token_t *)kmalloc(sizeof(token_t), GFP_KERNEL);            \
	tt->tt_data = (char *)kmalloc(max_length, GFP_KERNEL);           \
	tt->tt_size = 0;                                                 \
	tt->next = NULL;                                                 \
	if (tt == NULL || tt->tt_data == NULL)

#define aud_buff_append_normal(prototype, datum, tt)                     \
        *((prototype *)(tt->tt_data + tt->tt_size)) = datum;             \
	tt->tt_size += sizeof (prototype)

#define aud_buff_append_by_ptr(size, data_p, tt)                         \
        memcpy (tt->tt_data + tt->tt_size, data_p, size);                \
	tt->tt_size += size

#define aud_buff_append_n_data(length, data_p, tt)                       \
        memcpy(tt->tt_data + tt->tt_size, data_p, length);               \
	tt->tt_size += length

#define aud_token_size ()                                                \
        tt->size

/**
 *  token generation functions
 */
#ifdef __STDC__

extern int args_to_string(int, char **, char **, int *);
extern token_t *aud_to_arg(int, char **);
extern token_t *aud_to_attr(struct inode *);
extern token_t *aud_to_exec_args(int argc, char **);
extern token_t *aud_to_exec_env(int, char **);
//extern token_t *aud_to_exit(int, int);
extern token_t *aud_to_header(int, int);
extern token_t *aud_to_in_addr(struct in_addr *);
extern token_t *aud_to_ip(struct in_addr *);
extern token_t *aud_to_ipc(int);
extern token_t *aud_to_ipc_perm(struct ipc_perm *);
extern token_t *aud_to_iport(u_short);
extern token_t *aud_to_newgroups(gid_t *);
extern token_t *aud_to_opaque(const char *, short);
extern token_t *aud_to_path(const char *);
extern token_t *aud_to_process(unsigned int, unsigned short, unsigned short,
			       unsigned short, unsigned short, int, int,
			       aud_tid_t *);
extern token_t *aud_to_return(int, int);
extern token_t *aud_to_seq(long);
extern token_t *aud_to_socket(struct socket *);
extern token_t *aud_to_socket_inet(struct socket *);
extern token_t *aud_to_subject(unsigned int, unsigned short, unsigned short,
	 		       unsigned short, unsigned short, int, int,
	 		       aud_tid_t *);
extern token_t *aud_to_text(const char *);
extern token_t *aud_to_args_m(int, char *, int);
extern token_t *aud_to_args_n(int, int, char *);
extern token_t *aud_to_trailer(int);
extern token_t *aud_to_comm(char *);
extern token_t *aud_to_parent(int, char *);
extern token_t *aud_to_unix_domain(struct socket *);
#ifdef LBSM_SUPPORT_SE_LINUX
extern token_t *aud_to_label(unsigned short, security_id_t);
#endif

#else

extern int args_to_string();
extern token_t *aud_to_arg();
extern token_t *aud_to_attr();
extern token_t *aud_to_exec_args();
extern token_t *aud_to_exec_env();
extern token_t *aud_to_exit();
extern token_t *aud_to_header();
extern token_t *aud_to_in_addr();
extern token_t *aud_to_ip();
extern token_t *aud_to_ipc();
extern token_t *aud_to_ipc_perm();
extern token_t *aud_to_iport();
extern token_t *aud_to_newgroups();
extern token_t *aud_to_opaque();
extern token_t *aud_to_path();
extern token_t *aud_to_process();
extern token_t *aud_to_return();
extern token_t *aud_to_seq();
extern token_t *aud_to_socket();
extern token_t *aud_to_socket_inet();
extern token_t *aud_to_subject();
extern token_t *aud_to_text();
extern token_t *aud_to_args_m();
extern token_t *aud_to_args_n();
extern token_t *aud_to_trailer();
extern token_t *aud_to_comm();
extern token_t *aud_to_parent();
extern token_t *aud_to_unix_domain();
#ifdef LBSM_SUPPORT_SE_LINUX
extern token_t *aud_to_label();
#endif

#endif /* __STDC__ */

#else 

/**
 *  AUD_ARG token
 */
struct _aud_arg_tok_n {
	u_char	num;		/* number of arguments     */
	u_long	val;		/* value of arguments      */
	u_short length;		/* length of argument data */
	char *	data;		/* argument data           */
};
typedef struct _aud_arg_tok_n aud_arg_tok_t;

/**
 *  AUD_ATTR token
 */
struct _aud_attr_tok_n {
	u_long	mode;		/* mode of attribute       */
	u_long	uid;		/* user id                 */
	u_long	gid;		/* group id                */
	long	fs;		/* number of file system   */
	long	node;		/* inode number            */
	u_long	dev;		/* device number           */
};
typedef struct _aud_attr_tok_n aud_attr_tok_t;

/**
 *  AUD_DATA token
 */
struct _aud_data_tok_n {
	u_char	pfmt;		/* data format             */
	u_char	size;		/* data size               */
	u_char	number;		/*                         */
	char *	data;		/* data                    */
};
typedef struct _aud_data_tok_n aud_data_tok_t;

/**
 *  AUD_EXIT token
 */
struct _aud_exit_tok_n {
	long	status;		/* exist status            */
	long	retval;		/* return value            */
};
typedef struct _aud_exit_tok_n aud_exit_tok_t;

/**
 *  AUD_FILE token
 */
struct _aud_file_tok_n {
	time_t	time;		/* file creatig or closing time */ 
	time_t	msec;            
	u_short	length;		/* length of file mame     */
	char *	fname;		/* file name               */
};
typedef struct _aud_file_tok_n aud_file_tok_t;

/**
 *  AUD_GROUPS token
 */
struct _aud_groups_tok_n {
	gid_t	groups[NGROUPS_MAX]; /* groups               */
};
typedef struct _aud_groups_tok_n aud_groups_tok_t;

/**
 *  AUD_HEADER token
 */
struct _aud_header_tok_n {
	u_long		length;	/* length of audit record  */
	u_char		version;/* version of audit system */
	aud_event_t	event;	/* event ID                */
	u_short		emode;	/* event mode              */
	time_t		time;	/* event time              */
	time_t		msec;
};
typedef struct _aud_header_tok_n aud_header_tok_t;

/**
 *  AUD_INADDER token 
 */
struct _aud_inaddr_tok_n {
	/*struct in_addr ia;*/	
	long	s_addr;		/* internet address        */
};
typedef struct _aud_inaddr_tok_n aud_inaddr_tok_t;

/**
 *  AUD_IP token
 *
 *  ** CAUTION **
 *  This structure is different from the Sun BSM. In the BSM,
 *  it is used "struct ip" data type for soecifying IP.
 */
struct _aud_ip_tok_n {
	u_char	version;	/* version for IP          */
	//struct ip  ip;	/* Sun BSM                 */
	u_long	ip;		/* internet address        */
};
typedef struct _aud_ip_tok_n aud_ip_tok_t;

/**
 *  AUD_IPC token
 */
struct _aud_ipc_tok_n {
	u_char	type;		/* type of IPC object      */
	key_t	id;		/* id of IPC object        */
};
typedef struct _aud_ipc_tok_n aud_ipc_tok_t;

/**
 *  AUD_IPC_PERM token
 */ 
struct _aud_ipc_perm_tok_n {
	uid_t	huid;		/* owner's user ID         */
	gid_t	hgid;		/* oener's group ID        */
	uid_t	cuid;		/* creator's user ID       */
	gid_t	cgid;		/* creator's group ID      */
	long	ipc_mode;	/* ipc mode                */
	long	seq_num;	/* sequence number         */
	long	ipc_key; 	/* ipc key                 */
};
typedef struct _aud_ipc_perm_tok_n aud_ipc_perm_tok_t;

/**
 *  AUD_IPORT token
 */
struct _aud_iport_tok_n {
	u_short	iport;		/* port for IP		   */
};
typedef struct _aud_iport_tok_n aud_iport_tok_t;

/**
 *  AUD_INVALID token
 */
struct _aud_invalid_tok_n {
	u_short	length;		/* length of data          */
	char *	data;		/* data                    */
};
typedef struct _aud_invalid_tok_n aud_invalid_tok_t;

/**
 *  AUD_OPAQUE token 
 */
struct _aud_opaque_tok_n {
	u_short	length;		/* length of data          */
	char *	data;		/* data                    */
};
typedef struct _aud_opaque_tok_n aud_opaque_tok_t;

/** 
 *  AUD_PATH token
 */
struct _aud_path_tok_n {
	u_short	length;		/* length of the path name */
	char *	name;		/* path name               */
};
typedef struct _aud_path_tok_n aud_path_tok_t;

/**
 *  AUD_PROC token 
 */
struct _aud_proc_tok_n {
	aud_id_t	auid;	/* audit ID                */
	uid_t		euid;	/* user ID                 */
	gid_t		egid;	/* group ID 		   */
	uid_t		ruid;	/* real user ID            */
	gid_t		rgid;	/* real group ID           */
	pid_t		pid;	/* process ID              */
	pid_t		sid;	/* session ID		   */
	aud_tid_t	tid;	/* terminal ID             */
};
typedef struct _aud_proc_tok_n aud_proc_tok_t;

/**
 *  AUD_RET token
 */
struct _aud_ret_tok_n {
	u_char	error;		/* process error code      */
	u_long	retval;		/* return value 	   */
};
typedef struct _aud_ret_tok_n aud_ret_tok_t;

/**
 *  AUD_SEQ token
 */
struct _aud_seq_tok_n {
	u_long	num;		/* sequence number         */
};
typedef struct _aud_seq_tok_n aud_seq_tok_t;

/**
 *  AUD_SOCKET token
 */
struct _aud_socket_tok_n {
	short	type;		/* socket type	           */
	u_short	lport;		/* local port number       */
	u_long	laddr;		/* local IP address        */
	u_short	fport;		/* remote port number      */
	u_long	faddr;		/* remote IP address       */
};
typedef struct _aud_socket_tok_n aud_socket_tok_t;

/**
 *  AUD_SOCKET_INET token 
 */
struct _aud_socket_inet_tok_n {
	short	type;		/* socket family           */
	u_short	lport;		/* local port number       */
	u_long	laddr;		/* socket address          */
};
typedef struct _aud_socket_inet_tok_n aud_socket_inet_tok_t;

/**
 *  AUD_SUBJECT token
 */ 
struct _aud_subj_tok_n {
	aud_id_t		auid;	/* audit ID                */
	uid_t		euid;	/* user ID                 */
	gid_t		egid;	/* group ID                */
	uid_t		ruid;	/* real user ID            */
	gid_t		rgid;	/* real group ID           */
	pid_t		pid;	/* process ID              */
	pid_t		sid;	/* session ID              */
	aud_tid_t	tid;	/* termical ID             */
};
typedef struct _aud_subj_tok_n  aud_subj_tok_t;

/**
 *  AUD_SERVER token
 */
struct _aud_server_tok_n {
	aud_id_t	auid;	/* audit ID 		   */
	uid_t		euid;	/* user ID                 */
	uid_t		ruid;	/* real user ID            */
	gid_t		egid;	/* group ID		   */
	pid_t		pid;	/* process ID		   */
};
typedef struct _aud_server_tok_n aud_server_tok_t;

/**
 *  AUD_TEXT token
 */
struct _aud_text_tok_n {
	u_short	length;		/* length of data          */
	char *	data;		/* data                    */
};
typedef struct _aud_text_tok_n aud_text_tok_t;

/**
 *  AUD_TRAILER token
 */
struct _aud_trailer_tok_n {
	u_short	magic;		/* pad number              */
	u_long	length;		/* length of audit record  */
};
typedef struct _aud_trailer_tok_n aud_trailer_tok_t;

/**
 *  AUD_COMM token 
 *
 *  This struct is LBSM original struct. In the BSM does not have this 
 *  struct. We introduce this token to get process's command name.
 */
struct _aud_comm_tok_n {
	u_short	length;		/* length of command name  */
	char *	command;	/* command name            */
};
typedef struct _aud_comm_tok_n aud_comm_tok_t;

/**
 *  AUD_PARENT token
 *
 *  This struct is LBSM original struct. In the BSM does not have this 
 *  struct. We introduce this token to get parent's process information.
 */
struct _aud_parent_tok_n {
	pid_t	pid;		/* parent' process ID      */
	u_short	length;		/* length of parent's process name */
	char *	command;	/* parent's process name   */
};
typedef struct _aud_parent_tok_n aud_parent_tok_t;

/**
 *  AUD_EXEC_ARG token
 */
struct _aud_exec_arg_tok_n {
	short	num;		/* number of arguments     */
	char **	args;		/* arguments list          */
};
typedef struct _aud_exec_arg_tok_n aud_exec_arg_tok_t;

/**
 *  AUD_EXEC_ENV token
 */
struct _aud_exec_env_tok_n {
	short	num;		/* number of environments */
	char **	args;		/* environments list      */
};
typedef struct _aud_exec_env_tok_n aud_exec_env_tok_t;

/**
 *  AUD_UNIX_DOMAIN token
 */
struct _aud_unix_domain_tok_n {
	short	type;		/* domain type	STREAM or DATAGRAM */
	long	node;		/* inode number            */
	u_short	length;		/* length of the path name */
	char *	name;		/* path name               */
};
typedef struct _aud_unix_domain_tok_n aud_unix_domain_tok_t;

#ifdef LBSM_SUPPORT_SELINUX

/**
 *  AUD_LABEL token
 */
struct _aud_label_tok_n {
	u_short	type;			/* label type              */
	u_short	length;			/* length of the context   */
	security_context_t context;	/* security context        */
};
typedef struct _aud_label_tok_n aud_label_tok_t;

#endif /* LBSM_SUPPORT_SELINUX */

/*
 *  AUD_TOKEN general structure.
 */       
struct _aud_token_n {
	char			id;	/* token ID		  */
	struct _aud_token_n *	next;   /* pointer for next token */
	struct _aud_token_n *	prev;   /* pointer for prev token */
	char *			data;   /* for DEBUG              */
	u_short 		size;   /* for DEBUG              */
	union {			    	/* token data             */
		aud_arg_tok_t		arg;
		aud_attr_tok_t		attr;
		aud_data_tok_t		data;
		aud_exit_tok_t		exit;
		aud_file_tok_t		file;
		aud_groups_tok_t	groups;
		aud_header_tok_t	header;
		aud_inaddr_tok_t	inaddr;
		aud_ip_tok_t		ip;
		aud_ipc_perm_tok_t	ipc_perm;
		aud_ipc_tok_t		ipc;
		aud_iport_tok_t		iport;
		aud_invalid_tok_t	invalid;
		aud_opaque_tok_t	opaque;
		aud_path_tok_t		path;
		aud_proc_tok_t		proc;
		aud_ret_tok_t		ret;
		aud_server_tok_t	server;
		aud_seq_tok_t		seq;
		aud_socket_tok_t	socket;
		aud_socket_inet_tok_t	inet;
		aud_subj_tok_t		subj;
		aud_text_tok_t		text;
		aud_trailer_tok_t	trailer;
		aud_exec_arg_tok_t	exec_arg;
		aud_exec_env_tok_t	exec_env;
		aud_comm_tok_t		comm;
		aud_parent_tok_t	parent;
		aud_unix_domain_tok_t	unix_domain;
#ifdef LBSM_SUPPORT_SELINUX
		aud_label_tok_t		label;
#endif /* LBSM_SUPPORT_SELINUX */
	} un;
};
typedef struct _aud_token_n aud_token_t;

#endif /* __KERNEL__ */

#ifdef __cplusplus
}
#endif

#endif  /* !_LBSM_AUDIT_TOKEN_H */
