/*
 *
 *  Linux Basic Security Modules 
 *  LBSM project team. <lbsm@sec.jri.co.jp>
 *
 *  Copyright (c) 2002-2004 The Japan Research Institute, Limited
 *  http://www.jri.co.jp/security/lbsm/
 *
 *  This is a part of Linux Basic Security Modules.
 *
 *  This programme is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This programme is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free software Foundation, Inc.,
 *  59 Temple Place, suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _LBSM_LIBRARY_H
#define _LBSM_LIBRARY_H

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <audit.h>
#include <audit_token.h>

#ifdef __cplusplus
extern "C" {
#endif

#define MAX_BUF			1024

/**
 *  Basic Data type
 */
#define aud_mask_v		unsigned long
#define aud_id_t		int

/**
 *  System audit files for auditd 
 */
#define AUDITCLASSFILE_XML	LBSM_DIR"/audit_class.xml"
#define AUDITCONTROLFILE_XML	LBSM_DIR"/audit_control.xml"
#define AUDITEVENTFILE_XML	LBSM_DIR"/audit_event.xml"
#define AUDITUSERFILE_XML	LBSM_DIR"/audit_user.xml"
#define AUDITDATAFILE		LBSM_DIR"/audit_data"

/**
 *   Array sizes for audit library structures
 */
#define AUD_CLASS_NAME_MAX	8	/* max size of class name                   */
#define AUD_CLASS_DESC_MAX	72	/* max size of class description            */ 
#define AUD_EVENT_NAME_MAX	30	/* max size of event name                   */
#define AUD_EVENT_DESC_MAX	50	/* max size of event description            */

/**
 *  Error codes
 */ 
#ifndef __ERROR_STATE__
#define __ERROR_STATE__
enum ERROR_STATE {
	LBSM_SUCC,			/* sucess                                   */
	LBSM_EOPF,			/* unable to open file                      */
	LBSM_ECLS,			/* unable to close file                     */
	LBSM_EALM,			/* unable to allocate enough memory         */
	LBSM_EINVA,			/* invarid argument                         */
	LBSM_EEVN,			/* unable to get environment variable       */
	LBSM_EINIT,			/* unable to initialize                     */
	LBSM_EAUDIT,			/* standerd error on audit                  */
	LBSM_EREAD,			/* unable to read data                      */
	LBSM_EOF,			/* audit trail end of file                  */
	LBSM_EEVT,			/* invarid token ID                         */
	LBSM_ERR			/* others                                   */ 
};
#endif /* !__ERROR_STATE__ */

/**
 *  print mode for "praudit" command 
 */
enum PRINT_MODE {
	AUD_NORMAL_TYPE,		/* normal print mode                        */
	AUD_SYSTEM_TYPE,		/* hex print mode                           */
	AUD_ROW_TYPE,			/* binary print mode                        */
	AUD_XML_TYPE			/* xml print mode                           */
};

/**
 *  Default entry name (audit_control's entry)
 */
#define  DIR_ENTRY		"dir"		/* DIR entry name                   */
#define  FLAGS_ENTRY		"flags"		/* FLAGS entry name                 */
#define  NAFLAGS_ENTRY		"naflags"	/* NFLAGS entry name                */
#define  MINFREE_ENTRY		"minfree"	/* MINFREE entry name               */
#define  FORWARDER_ENTRY	"forwarder"	/* FORWARDER entry name             */

/**
 *  audit config file types
 */
typedef enum {
	AUD_FILE_EVENT=0,
	AUD_FILE_CLASS,
	AUD_FILE_CONTROL,
	AUD_FILE_USER,
	AUD_FILE_SUBJECT,
	AUD_FILE_OBJECT,
	AUD_FILE_DOMAIN
} aud_parse_file_t;

/**
 *  audit instance
 */
struct _aud_conf_instance_n {
	aud_parse_file_t	type;
	int 			status;
	union {
		struct _aud_event_n *	     aud_event;
		struct _aud_class_n *	     aud_class;
		struct _aud_user_n *	     aud_user;
		struct _aud_control_n *	     aud_control;
		struct _mac_control_list_n * mac_subject;
		struct _mac_control_list_n * mac_object;
		struct _mac_domain_list_n *  mac_domain;
	} aud_list;
};
typedef struct _aud_conf_instance_n aud_conf_instance_t;

/**
 *  audit instance cache node
 */
struct _aud_conf_cache_node_n {
	union {
		struct _aud_event_n *	     aud_event;
		struct _aud_class_n *	     aud_class;
		struct _aud_user_n *	     aud_user;
		struct _aud_control_n *	     aud_control;
		struct _mac_control_list_n * mac_subject;
		struct _mac_control_list_n * mac_object;
		struct _mac_domain_list_n *  mac_domain;
	} aud_list;
	struct stat file_stat;
};
typedef struct _aud_conf_cache_node_n aud_conf_cache_node_t;

struct _aud_conf_cache_n {
	aud_conf_cache_node_t *cached_aud_event;	/* cached audit event data   */
	aud_conf_cache_node_t *cached_aud_class;	/* cached audit class data   */
	aud_conf_cache_node_t *cached_aud_user;   	/* cached audit user data    */
	aud_conf_cache_node_t *cached_aud_control;	/* cached audit control data */ 
	aud_conf_cache_node_t *cached_mac_subject;	/* cached mac subject data   */ 
	aud_conf_cache_node_t *cached_mac_object;	/* cached mac object data    */ 
	aud_conf_cache_node_t *cached_mac_domain;	/* cached mac domain data    */ 
};
typedef struct _aud_conf_cache_n aud_conf_cache_t;

/**
 *   For audit_class(5)
 */
struct _audit_class_n {
	aud_mask_v		mask;		/* class mask                       */
	char *			name;		/* class name                       */
	char *			desc;		/* class description                */
	struct _audit_class_n * next;		/* pointer of next class structure  */
};
typedef struct _audit_class_n AUDIT_CLASS;

/**
 *  For audit_event(5) 
 */
struct _audit_event_n {
	int			number;		/* number of event                  */
	char *			name;		/* event name                       */
	char *			desc;		/* event description                */
	int			classN;		/* a number of class                */
	char **			classes;	/* classes                          */
	aud_mask_v		mask;		/* class mask                       */
	struct _audit_event_n *	next;		/* pointer of next event structure  */
};
typedef struct _audit_event_n AUDIT_EVENT;

/**
 *  For audit_control(5)
 */
struct _audit_dir_n {
	char *			dirname;	/* directory name                   */
	struct _audit_dir_n *	next;		/* pointer of next entry            */
};
typedef struct _audit_dir_n S_DIR;

struct _audit_control_n {
	S_DIR *			dir;		/*  audit directory                 */
	char *			flags;		/*  audit flags                     */
	int			minfree;	/*  free space parcentage           */
	char *			naflags;	/*  not audit flags                 */
	int			dir_num;	/*  number of directory             */
	int			pos;		/*  position                        */
	char *			forwarder;	/*  forwarder                       */
};
typedef struct _audit_control_n AUDIT_CONTROL;

/**
 *  For audit_user(5)
 */
struct _audit_user_n {
	char *			aud_name;	/*  name of audit user              */
	char *			aud_flags;	/*  audit flags                     */
	char *			aud_naflags;	/*  not audit flags                 */
	aud_mask_v		aud_always;	/*  always mask                     */
	aud_mask_v		aud_never;	/*  never mask                      */
	struct _audit_user_n *	next;		/*  next pointer                    */
};
typedef struct _audit_user_n AUDIT_USER;

/**
 *  collection structure for aud_event
 */
struct _aud_event_n {
	AUDIT_EVENT *		start;	/* start pointer for list of aud_event      */
	int			num;	/* number of element                        */
	int			pos;	/* current position                         */
	int			state;	/* status : if set list, status 0, other -1 */
};

/**
 *  collection structure for aud_class
 */
struct _aud_class_n {
	AUDIT_CLASS *		start;	/* start pointer for list of aud_class      */
	int			num;	/* number of element                        */
	int			pos;	/* current position                         */
	int			state;	/* status : if set list, status 0, other -1 */
};

/**
 *  collection structure for aud_control
 */
struct _aud_control_n {
	AUDIT_CONTROL *		this;	/* start pointer for list of aud_control    */
	AUDIT_CONTROL *		is;	/* start pointer for list of aud_control    */
	int			state;	/* status : if set list, status 0, other -1 */
};

/**
 *  collection structure for aud_user
 */
struct _aud_user_n {
	AUDIT_USER *		start;	/* start pointer for list of aud_user       */
	int			num;
	int			pos;
	int			state;	/* status : if set list, status 0, other -1 */
};

/**
 *   aud_record_row structure
 */
struct _aud_record_row_n {
	void *			rec_buff;	/* row data                         */
	int			rec_pos;	/* record position                  */
	int			rec_size;	/* record size                      */
};
typedef struct _aud_record_row_n  aud_record_row_t;

#ifndef _WRITE_MODE
#define _WRITE_MODE

/**
 *  write mode
 */
typedef enum _WRITE_MODE {
	ADD,				/* ADD mode                                 */
	DEL				/* DEL mode                                 */
} W_MODE;

#endif /* _WRITE_MODE */

/**
 *  Function that manipulate audit event
 */
#ifdef __STDC__

extern aud_conf_instance_t *setauevent(void);
extern int endauevent(aud_conf_instance_t *);
extern AUDIT_EVENT *getauevent(aud_conf_instance_t *);
extern AUDIT_EVENT *getauevnam(aud_conf_instance_t *, char *);
extern AUDIT_EVENT *getauevnum(aud_conf_instance_t *, int);
extern char *my_gets(char *, int, char **);

#else 

extern aud_conf_instance_t *setauevent();
extern int setauevent_t();
extern int endauevent();
extern AUDIT_EVENT *getauevent();
extern AUDIT_EVENT *getauevnam();
extern AUDIT_EVENT *getauevnum();
extern char *my_gets();

#endif /* __STDC__ */

/**
 *  Function that manipulate audit class
 */
#ifdef __STDC__

extern aud_conf_instance_t *setauclass(void);
extern int endauclass(aud_conf_instance_t *);
extern AUDIT_CLASS *getauclass(aud_conf_instance_t *);
extern AUDIT_CLASS *getauclassnam(aud_conf_instance_t *, char *);

#else

extern aud_instance_t *setauclass();
extern int endauclass();
extern AUDIT_CLASS *getauclass();
extern AUDIT_CLASS *getauclassnam();

#endif /* __STDC__ */

/**
 *  Function that manipulate audit control
 */
#ifdef __STDC__

extern aud_conf_instance_t *setac(void);
extern int endac(aud_conf_instance_t *);
extern char *getacdir(aud_conf_instance_t *);
extern int getacmin(aud_conf_instance_t *);
extern char *getacflg(aud_conf_instance_t *);
extern char *getacna(aud_conf_instance_t *);
extern char *getacforwarder(aud_conf_instance_t *);

#else

extern aud_conf_instance_t *setac();
extern int endac();
extern char *getacdir();
extern int getacmin();
extern char *getacflg();
extern char *getacna();
extern char *getacforwarder();

#endif /* __STDC__ */
 
/**
 *  Function that manipulate audit user
 */
#ifdef __STDC__

extern aud_conf_instance_t *setauuser(void);
extern int endauuser(aud_conf_instance_t *ains);
extern AUDIT_USER  *getauuserent(aud_conf_instance_t *ains);
extern AUDIT_USER  *getauusernam(aud_conf_instance_t *ains, char *);

#else

extern aud_conf_instance_t *setauuser();
extern int endauuser();
extern AUDIT_USER *getauuserent();
extern AUDIT_USER *getauusernam();

#endif

/**
 *  Functions that configurate kernel masks
 */
#ifdef __STDC__

extern int setDefaultEventMask(aud_conf_instance_t *, aud_conf_instance_t *);
extern int flagToMask(aud_conf_instance_t *c_ins, char *, aud_mask_t **);
extern int setAuditClassMask(aud_conf_instance_t *, aud_conf_instance_t *);
extern int setUserMask(aud_conf_instance_t *, aud_conf_instance_t *);
extern int set_audit_mask(void);

#else

extern int setDefaultEventMask();
extern int flagToMask();
extern int setAuditClassMask();
extern int setUserMask();
extern int set_audit_mask();

#endif /* __STDC__ */

/**
 *   main interface 
 */
#ifdef __STDC__

extern int set_audit_mask(void);

#else 

extern int set_audit_mask();

#endif  /* __STDC__ */

/**
 *  system call
 */
#ifdef __STDC__

extern int auditon(int mask, aud_mask_t *type, int value);
extern int auditsvc(int fd, int a);
extern int audituser(char *name);
extern int getaudit(auditinfo_t *ai);
extern int getauid(aud_id_t id);
extern int kernstate(aud_mask_v mask);
extern int getuseraudit(aud_id_t id, aud_mask_v);
extern int setaudit(auditinfo_t *ai);
extern int setauid(aud_id_t id);
extern int setkernstate(aud_mask_v mask);
extern int setuseraudit(aud_id_t id, aud_mask_v mask);
extern int auditstat(aud_stat_t *st);
extern int getaudit_t(auditinfo_t *ai, int pid);
extern int setmaclabel(void);

#else

extern int auditon();
extern int auditsvc();
extern int audituser();
extern int getaudit();
extern int getauid();
extern int kernstate();
extern int getuseraudit();
extern int setaudit();
extern int setauid();
extern int setkernstate();
extern int setuseraudit();
extern int auditstat();
extern int getaudit_t();
extern int setmaclabel();

#endif /* __STDC__ */

/**
 *  Function for manupulating audit trail
 */
#ifdef __STDC__

extern int aud_open_audit_trail(char *);
extern int aud_close_audit_trail(int);
extern int aud_read_row_rec(int, aud_record_row_t **);
extern void aud_free_row_rec(aud_record_row_t *);
extern void aud_switch_mode(void);

extern int aud_open_tok(aud_record_row_t *);
extern int aud_close_tok(void);
extern aud_token_t *aud_get_tok(void);
extern aud_token_t *aud_get_tok_n(int n);
extern aud_token_t *aud_get_tok_p(char type);
extern int aud_print_tok(FILE *fd, aud_token_t *, int, char *, int);
extern int aud_print_tok_xml(FILE *, aud_token_t *, int);

extern void put_char(aud_record_row_t *, char *);
extern void put_u_char(aud_record_row_t *, u_char *);
extern void put_bytes(aud_record_row_t *, int, char *);
extern void put_string(aud_record_row_t *, char **);
extern void put_string_n(aud_record_row_t *, int, char **);
extern void put_string_m(aud_record_row_t *, char **);
extern void put_short(aud_record_row_t *, short *);
extern void put_u_short(aud_record_row_t *, u_short *);
extern void put_int(aud_record_row_t *, int *);
extern void put_long(aud_record_row_t *, long *);
extern void put_u_long(aud_record_row_t *, u_long *);
extern void put_float(aud_record_row_t *, float *);
extern void put_double(aud_record_row_t *, double *);

#else

extern int aud_open_audit_trail()
extern int aud_close_audit_trail();
extern int aud_read_row_rec();
extern void aud_free_row_rec();
extern void aud_switch_mode();

extern int aud_open_tok();
extern int aud_close_tok();
extern aud_token_t *aud_get_tok();
extern aud_token_t *aud_get_tok_n();
extern aud_token_t *aud_get_tok_p();
extern int aud_print_tok();
extern int aud_print_tok_xml();

extern void put_char();
extern void put_u_char();
extern void put_bytes();
extern void put_string();
extern void put_short();
extern void put_u_short();
extern void put_int();
extern void put_long();
extern void put_u_long();
extern void put_float();
extern void put_double();

#endif

#ifdef __cplusplus
}
#endif

#endif  /* !_LBSM_LIBRARY_H */
